#!/bin/tcsh
#
# Usage:
# source backscalReset.csh SPECFILE MINAREA EVTFILE REFSPEC LOGFILE
#
# This script checks the current BACKSCAL setting of the input spectrum file
# SPECFILE to see whether it is larger than MINAREA.  If BACKSCAL is unreasonably
# small (i.e., smaller than MINAREA), then it is assumed that the value stored
# in the header of SPECFILE is some sort of dummy or placeholder and an attempt
# is made to reset it, copying the setting from REFSPEC (if provided) or
# determining it directly (making use of the bad pixel maps in the input events
# list EVTFILE).
#
# Verbose output is stored in LOGFILE while more terse comments are directed both
# there and to STDOUT.

echo " " >>! ${5}
echo "Now running backscalReset.csh with the following parameters:" >>! ${5}
echo "    SPECfile = ${1}" >>! ${5}
echo "    BACKSCAL threshold = ${2}" >>! ${5}
echo "    EVTfile = ${3}" >>! ${5}
echo "    Reference SPECfile = ${4}" >>! ${5}
echo "    LogFile = ${5}" >>! ${5}

set SPECname_back = ${1}
set MinArea_back = ${2}
set EVTfile_back = ${3}
set refSPEC_back = ${4}
set LogFile_back = ${5}

fkeypar ${SPECname_back} BACKSCAL
set tempBackScal2 = `pget fkeypar value`
set BackScalTest2 = `calc ${tempBackScal2} - ${MinArea_back} | awk '{print substr($0,1,1)}'`
# echo "><><>< The value of BACKSCAL for spectrum ${SPECname_back} was found to be ${tempBackScal2}" |& tee -a ${LogFile_back}
echo "    <> Initial BACKSCAL value is ${tempBackScal2}" |& tee -a ${LogFile_back}
#echo "                    BackScalTest2 is set to ${BackScalTest2}" |& tee -a ${LogFile_back}
if ("${BackScalTest2}" == "-") then
    # ie., if BACKSCAL is too small, because that might mean it hasn't been set...
    # echo "       This is suspiciously low; now using SAS routine BACKSCALE to reset it..." |& tee -a ${LogFile_back}
    echo "       This is suspiciously low; attempting to reset it..." |& tee -a ${LogFile_back}
    if (-e ${refSPEC_back}) then
	fkeypar ${refSPEC_back} BACKSCAL
	set refBackScal = `pget fkeypar value`
	fparkey $refBackScal $SPECname_back BACKSCAL comm="Scaling factor for background" add+
	echo "       BACKSCAL value of ${refBackScal} was copied from ${refSPEC_back}" |& tee -a ${LogFile_back}
	unset refBackScal
    else
	if (-e tmp.out) /bin/rm -f tmp.out
	backscale spectrumset=${SPECname_back} badpixlocation=${EVTfile_back} >>&! tmp.out
	cat tmp.out >>! ${LogFile_back}
	echo "       BACKSCALE warnings and errors:" |& tee -a ${LogFile_back}
	# grep "\*\*" tmp.out |& tee -a ${LogFile_back}
	grep -i " warning " tmp.out |& tee -a ${LogFile_back}
	grep -i " error " tmp.out |& tee -a ${LogFile_back}
	/bin/rm -f tmp.out
	fkeypar ${SPECname_back} BACKSCAL
	set tempBackScal2 = `pget fkeypar value`
	echo "       BACKSCAL has been reset to ${tempBackScal2}" |& tee -a ${LogFile_back}
    endif
endif

unset SPECname_back
unset MinArea_back
unset EVTfile_back
unset refSPEC_back
unset LogFile_back
unset tempBackScal2
unset BackScalTest2
