#!/bin/tcsh
#
# Usage:
# source chooseEVTlists.csh "INLIST"
#
# Given an input list of XMM events files (which *MUST* be enclosed in
# quotation marks), cycle through each and generate a list of just the
# observations made with a scientifically useful filter (Thin1, Thin2,
# Medium, Thick or Open), including only one file per exposure.
#
# The constructed list is stored in $chosenEVTlist

if (-e sas.log) then
    echo " " >>! sas.log
    echo "Now running chooseEVTlists.csh with the following parameters:" >>! sas.log
    echo "    EVTlist = ${1}" >>! sas.log
endif

set tmpEVTlist = ( $1 )
fkeypar ${tmpEVTlist[1]} INSTRUME
echo " "
echo "chooseEVTlists.csh for `pget fkeypar value`:"

set chosenEVTlist = ( )
set expIDlist
foreach ii ( ${tmpEVTlist} )
    fkeypar ${ii} FILTER
    set filterStr = `pget fkeypar value | tr -d "'" | tr -d '"' | tr '[a-z]' '[A-Z]' | awk '{print substr($0,1,3)}'`
    if (($filterStr == THI) || ($filterStr == MED) || ($filterStr == OPE)) then
	# If filter is usefull, add to list if exposure not already represented...
	fkeypar ${ii} EXPIDSTR
	set tmpIDstr = `pget fkeypar value | tr -d "'" | tr -d '"'`
	if (`echo "${expIDlist}"` == `echo "${expIDlist}" | sed -e "s/${tmpIDstr}/xx/g"`) then
	    # ...if current EXPIDSTR not already in list...
	    set chosenEVTlist = ( $chosenEVTlist ${ii} )
	    set expIDlist = "${expIDlist}""${tmpIDstr}"
	endif
    endif
end
unset tmpIDstr
unset filterStr
unset expIDlist
unset tmpEVTlist
