#!/bin/tcsh -x
#
# Usage:
# to provide values on the command line, these must be:
# ppsSpecPrep.csh DETID OBSID IAUNAME SRCNUM SDSS_RA SDSS_DEC COMMONNAME
# e.g.
# ppsSpecPrep.csh 014320 0305920101 "2XMM J010712.0+140844" 001 \
#            16.80014 14.14583 1RXSJ010712.9+140843
#
#############
# (1) Obtain/define variables
#

#debug# echo "Setting up variables"
#debug# echo " "

# Input variables
#
if ( "${1}x" != "x" ) then
    setenv DETID_PPS	${1}
else
    setenv DETID_PPS	014320
endif
if ( "${2}x" != "x" ) then
    setenv OBSID_PPS	${2}
else
    setenv OBSID_PPS	0305920101
endif
if ( "${3}x" != "x" ) then
    setenv IAUNAME_PPS	"${3}"
else
    setenv IAUNAME_PPS	"2XMM J010712.0+140844"
endif
if ( "${4}x" != "x" ) then
    setenv SRCNUM_PPS	${4}
else
    setenv SRCNUM_PPS	001
endif
if ( "${5}x" != "x" ) then
    setenv SDSS_RA	${5}
else
    setenv SDSS_RA	16.80014
endif
if ( "${6}x" != "x" ) then
    setenv SDSS_DEC	${6}
else
    setenv SDSS_DEC	14.14583
endif
if ( "${7}x" != "x" ) then
    setenv COMMONNAME	${7}
else
    setenv COMMONNAME	1RXSJ010712.9+140843
endif

#debug# echo " "
#debug# echo "Variables read in."
#debug# echo " "

# setenv PN_8_RATE	0.3037
# setenv M1_8_RATE	0.0757
# setenv M2_8_RATE	0.0791
# setenv PN_FILTER	Thin1
# setenv M1_FILTER	Thin1
# setenv M2_FILTER	Thin1
# setenv PN_SUBMODE	PrimeFullWindow
# setenv M1_SUBMODE	PrimeFullWindow
# setenv M2_SUBMODE	PrimeFullWindow
# setenv SDSS_REDSHIFT	0.07695

# Derived variables
#
setenv IAUNAME_TRUN \
   `echo ${IAUNAME_PPS} | tr -d 'i' | sed -e 's/2XMM J//g' | tr '+' 'p' | tr '-' 'm'`
# foreach i ($IAUNAME_PPS)
#    setenv IAUNAME_TRUN `echo ${i} | tr -d 'J' | tr '+' 'p' | tr '-' 'm'`
# end
setenv NAME_TRUN \
    `echo ${COMMONNAME} | awk '{if (substr($0,12,1) == ".") {print substr($0,1,11)} else {print substr($0,1,12)}}' | tr '+' 'p' | tr '-' 'm'`

setenv ANALYSIS_PATH "/Users/jonathan/Desktop/Projects/XMManalysis/"
setenv XSPEC_WORKDIR ${ANALYSIS_PATH}${DETID_PPS}_${IAUNAME_TRUN}_${NAME_TRUN}

#debug# echo " "
#debug# echo "Testing whether XSPEC_WORKDIR already exists"
#debug# echo " "

if (-e ${XSPEC_WORKDIR}) then
    # If a version of the xspec work directory already exists,
    # preserve it by appending "_v#" to its name.
    # 
    # Note: I tried to implement a check to only preserve the folder
    # if it is not empty, but the test for emptiness I was using
    # (saving the output of a ls command to a variable and then
    # checking the content of the variable) was problematic...
    #
    set tmpCounter = 0
    set preExistFolder = 1
    while (${preExistFolder} == 1)
        @ tmpCounter++
        set altWorkFolder = ${XSPEC_WORKDIR}"_v"${tmpCounter}
        if (! -e ${altWorkFolder}) set preExistFolder = 0
    end
    mv ${XSPEC_WORKDIR} ${altWorkFolder}
    mkdir ${XSPEC_WORKDIR}
    unset altWorkFolder
    unset preExistFolder
    unset tmpCounter
endif
mkdir ${XSPEC_WORKDIR}

#debug# echo "Work directory has been established."

#############
# (2) get ODF and PPS data
#

setenv fetchODF 0
setenv fetchPPS 0
if (! -e ${OBSID_PPS}/odf) setenv fetchODF 1
if (! -e ${OBSID_PPS}/pps) setenv fetchPPS 1
if (${fetchODF}+${fetchPPS} > 0) then
    if (! $?JDK_HOME) then
	setenv JDK_HOME /System/Library/Frameworks/JavaVM.framework/Versions/CurrentJDK/Home
	setenv JDK_HOME_SET
    endif
    if ($fetchODF == 0) then
	setenv levelStr " level=PPS"
    else if ($fetchPPS == 0) then
	setenv levelStr " level=ODF"
	else
	    setenv levelStr ""
	endif
    endif
    aioclient -S xsa.esac.esa.int -P 2002 -L \"GET obsno=${OBSID_PPS}${levelStr}\"
    unsetenv levelStr
    if ($?JDK_HOME_SET) then
	unsetenv JDK_HOME
	unsetenv JDK_HOME_SET
    endif
    setenv fileTest `/bin/ls AIOSOCK*.tar`
    if (${fileTest}x != x) then
	tar xf AIOSOCK*.tar
	/bin/rm AIOSOCK*.tar
    else
	echo "**** ERROR!  No data products downloaded for ${OBSID_PPS}! ****"
    endif
    unsetenv fileTest
    if ($fetchPPS == 0) then
	# if only obtaining ODF data, make sure tar ball is nested within OBSID_PPS/odf
	if (! -e ./${OBSID_PPS}) mkdir ${OBSID_PPS}
	if (! -e ./${OBSID_PPS}/odf) mkdir ${OBSID_PPS}/odf
	mv ${OBSID_PPS}.tar.gz ${OBSID_PPS}/odf/
    endif
    if ($fetchODF == 1) then
	# if ODF files amongst new data, expand and remove tarballs
	pushd ${OBSID_PPS}/odf/
	tar xfz ${OBSID_PPS}.tar.gz
	/bin/rm ${OBSID_PPS}.tar.gz
	tar xf *${OBSID_PPS}.TAR
	/bin/rm ${OBSID_PPS}.TAR
	popd
    endif
endif
unsetenv fetchODF
unsetenv fetchPPS


#############
# (2a) change directories and make sure SAS is set up to work with
# these data
#
pushd ${OBSID_PPS}/pps/

setenv PN_EVT `/bin/ls P${OBSID_PPS}PN*EVLI*FTZ`
setenv M1_EVT `/bin/ls P${OBSID_PPS}M1*EVLI*FTZ`
setenv M2_EVT `/bin/ls P${OBSID_PPS}M2*EVLI*FTZ`

gunzip *.FIT.gz

if (${PN_EVT}x != x) then
    set WORDLIST=(`echo $PN_EVT | sed 's/PN/P/g' | tr 'P' ' '`)
    setenv PN_OBSNUM $WORDLIST[2]
    unset WORDLIST
else
    echo "**** Warning!  There is no PPS PN events list!!! ****"
endif

if (${M1_EVT}x != x) then
    set WORDLIST=(`echo $M1_EVT | sed 's/M1/M/g' | tr 'M' ' '`)
    setenv M1_OBSNUM $WORDLIST[2]
    unset WORDLIST
else
    echo "**** Warning!  There is no PPS M1 events list!!! ****"
endif

if (${M2_EVT}x != x) then
    set WORDLIST=(`echo $M2_EVT | sed 's/M2/M/g' | tr 'M' ' '`)
    setenv M2_OBSNUM $WORDLIST[2]
    unset WORDLIST
else
    echo "**** Warning!  There is no PPS M2 events list!!! ****"
endif

if (! $?XMMSASSET) setsas
#debug#  if (! $?XMMSASSET) then
#debug#      echo "SAS appears to not be set up."
#debug#      echo "Has the setsas alias been defined?  Here is what which finds:"
#debug#      which setsas
#debug#      echo "The alias setsas will now be invoked."
#debug#      setsas
#debug#  endif
updatesasccf
setenv SAS_VERBOSITY 1
setsnv SAS_SUPPRESS_WARNING 1
echo "* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *" >>! sas.log
echo "* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *" >>! sas.log
echo "* * * SAS data prep done on  `date` * * *" >>! sas.log
echo "* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *" >>! sas.log
echo "* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *" >>! sas.log
setsas ../odf |& tee -a sas.log


#############
# (3) check whether previously-extracted spectra exist for all three
# detectors
#
setenv PNSPEC `/bin/ls P${OBSID_PPS}PN*SRSPEC0${SRCNUM_PPS}.FTZ`
setenv M1SPEC `/bin/ls P${OBSID_PPS}M1*SRSPEC0${SRCNUM_PPS}.FTZ`
setenv M2SPEC `/bin/ls P${OBSID_PPS}M2*SRSPEC0${SRCNUM_PPS}.FTZ`


#############
# (4) visually inspect extraction regions to look for contaminating
# sources in either the source or background extraction regions
#
# First verify that full-band images exist.  If they do not, then
# extract them.  If no viable data, skip this instrument and proceed.
#
# If source extraction region doesn't already exist, define one based
# upon SDSS coordinates and list of all detected 2XMM sources.
#
# Overlay extraction regions upon field image in DS9.  Save snapshot
# for later inspection.
#
# Test whether source is covered by instrument: use Funtools to get
# net counts?  Just get central pixel value from PPS exposure map?  If
# not, flag as not covered and do not attempt to proceed further with
# this instrument.


#############
# (5) Repeat step (4) showing just the soft-band PN image with source
# regions overlaid.  Save snapshot for later inspection.
#
# Automate effort to identify soft background noise: if source is
# sufficiently close to RawY=0, then measure low-energy counts per
# pixel at the high- and low-RawY ends of the background region.
# Significant difference?  Then set PN soft background noise flag.
# 
# Any time a source has too low a value of RawY, a grade 0 PN spectrum
# should be extracted so that it would be available later if the user
# wishes.


#############
# (6) check count rates for pileup thresholds.
#
# Compare PN_8_RATE, M1_8_RATE, and M2_8_RATE to thresholds (as
# function of observing mode) and set pileup flags accordingly. 
# Two flags: one for pileup in double events, one for pileup in
# singles.  If the latter is set, then the former is automatically
# also set.


#############
# (7) Spectra (re-)extractions
# 
if (${PNSPEC}x == x) then
    # If no PN spectrum was found in (3), one will have to be extracted...
endif
if (${M1SPEC}x == x) then
    # If no M1 spectrum was found in (3), one will have to be extracted...
endif
if (${M2SPEC}x == x) then
    # If no M2 spectrum was found in (3), one will have to be extracted...
endif
#
# No automatic triggers for extraction field contamination (4)?
#
# PN soft background noise (5):
# Automatically extract a pattern == 0 PN spectrum if RawY value is
# too low.  Rely upon PN soft background noise flag to determine
# whether this will be used in the pipeline or just readied for the
# user.
#
# If any pileup thresholds (6) are hit, then extract the necessary
# pattern == 0 and possibly also annular spectra.


#############
# (8) Define extraction regions.
#
# This is now done as part of diagnostic step 4, if one doesn't
# already exist.


#############
# (9) Determine flare background GTIs.
#
setenv PN_FLARE_LC `/bin/ls P${OBSID_PPS}PN*FBKTSR0000.FTZ`
setenv M1_FLARE_LC `/bin/ls P${OBSID_PPS}M1*FBKTSR0000.FTZ`
setenv M2_FLARE_LC `/bin/ls P${OBSID_PPS}M2*FBKTSR0000.FTZ`

if (${PN_FLARE_LC}x == x) then
    # if PN flare light-curve doesn't exist, one will have to be
    # extracted, and PN_FLARE_LC redefined.
endif

setenv PN_FLARE_GTI `echo ${PN_FLARE_LC} | sed -e 's/FBKTSR/FBKGTI/g' -e 's/FTZ/FIT/g'`
setenv M1_FLARE_GTI `echo ${M1_FLARE_LC} | sed -e 's/FBKTSR/FBKGTI/g' -e 's/FTZ/FIT/g'`
setenv M2_FLARE_GTI `echo ${M2_FLARE_LC} | sed -e 's/FBKTSR/FBKGTI/g' -e 's/FTZ/FIT/g'`

if ((-e ${PN_FLARE_LC}) && (! -e ${PN_FLARE_GTI})) then
    tabgtigen table=${PN_FLARE_LC} expression='RATE<10' gtiset=${PN_FLARE_GTI}
endif
if ((-e ${M1_FLARE_LC}) && (! -e ${M1_FLARE_GTI})) then
    tabgtigen table=${M1_FLARE_LC} expression='RATE<10' gtiset=${M1_FLARE_GTI}
endif
if ((-e ${M2_FLARE_LC}) && (! -e ${M2_FLARE_GTI})) then
    tabgtigen table=${M2_FLARE_LC} expression='RATE<10' gtiset=${M2_FLARE_GTI}
endif


#############
# (10) (Re-)extract spectra
#
# ...only if necessary/useful.


#############
# (10a) Generate diagnostic figures
#
# epatplot?
# ratios of all-event, singles-only, and anulus-only spectra?


#############
# (11) Generate response matrices
#
setenv PN_RMF `echo $PNSPEC | sed -e 's/SRSPEC/SR_RMF/g' -e 's/FTZ/FIT/g'`
setenv M1_RMF `echo $M1SPEC | sed -e 's/SRSPEC/SR_RMF/g' -e 's/FTZ/FIT/g'`
setenv M2_RMF `echo $M2SPEC | sed -e 's/SRSPEC/SR_RMF/g' -e 's/FTZ/FIT/g'`
setenv PN_ARF `echo $PNSPEC | sed -e 's/SRSPEC/SR_ARF/g' -e 's/FTZ/FIT/g'`
setenv M1_ARF `echo $M1SPEC | sed -e 's/SRSPEC/SR_ARF/g' -e 's/FTZ/FIT/g'`
setenv M2_ARF `echo $M2SPEC | sed -e 's/SRSPEC/SR_ARF/g' -e 's/FTZ/FIT/g'`

if (${PNSPEC}x != x  &&  ${PN_EVT}x != x) then
    if (! -e $PN_RMF) rmfgen spectrumset=${PNSPEC} rmfset=${PN_RMF} |& tee -a sas.log
    if (! -e $PN_ARF) \
	arfgen spectrumset=${PNSPEC} arfset=${PN_ARF} badpixlocation=${PN_EVT} \
	detmaptype=psf withrmfset=yes rmfset=${PN_RMF} |& tee -a sas.log
endif

if (${M1SPEC}x != x  &&  ${M1_EVT}x != x) then
    if (! -e $M1_RMF) rmfgen spectrumset=${M1SPEC} rmfset=${M1_RMF} |& tee -a sas.log
    if (! -e $M1_ARF) \
	arfgen spectrumset=${M1SPEC} arfset=${M1_ARF} badpixlocation=${M1_EVT} \
	detmaptype=psf withrmfset=yes rmfset=${M1_RMF} |& tee -a sas.log
endif

if (${M2SPEC}x != x  &&  ${M2_EVT}x != x) then
    if (! -e $M2_RMF) rmfgen spectrumset=${M2SPEC} rmfset=${M2_RMF} |& tee -a sas.log
    if (! -e $M2_ARF) \
	arfgen spectrumset=${M2SPEC} arfset=${M2_ARF} badpixlocation=${M2_EVT} \
	detmaptype=psf withrmfset=yes rmfset=${M2_RMF} |& tee -a sas.log
endif


#############
# (12) Run GRPPHA to prep the data for XSPEC
#
setenv PN_BGD `echo $PNSPEC | sed -e 's/SRSPEC/BGSPEC/g'`
setenv M1_BGD `echo $M1SPEC | sed -e 's/SRSPEC/BGSPEC/g'`
setenv M2_BGD `echo $M2SPEC | sed -e 's/SRSPEC/BGSPEC/g'`
setenv PN_SP1 `echo $PNSPEC | sed -e 's/SRSPEC/SR_SP_/g' -e 's/FTZ/FIT/g'`
setenv M1_SP1 `echo $M1SPEC | sed -e 's/SRSPEC/SR_SP_/g' -e 's/FTZ/FIT/g'`
setenv M2_SP1 `echo $M2SPEC | sed -e 's/SRSPEC/SR_SP_/g' -e 's/FTZ/FIT/g'`
setenv PN_SP2 `echo $PNSPEC | sed -e 's/SRSPEC/SR_SPG/g' -e 's/FTZ/FIT/g'`
setenv M1_SP2 `echo $M1SPEC | sed -e 's/SRSPEC/SR_SPG/g' -e 's/FTZ/FIT/g'`
setenv M2_SP2 `echo $M2SPEC | sed -e 's/SRSPEC/SR_SPG/g' -e 's/FTZ/FIT/g'`

if (${PNSPEC}x != x) then 
    if (! -e $PN_SP1) grppha $PNSPEC $PN_SP1 "chkey BACKFILE $PN_BGD & chkey RESPFILE $PN_RMF & chkey ANCRFILE $PN_ARF & bad 0-40 & good 41-80 & bad 1576-1646 & exit"
    if (! -e $PN_SP2) grppha $PN_SP1 $PN_SP2 "group min 25 & exit"
endif
if (${M1SPEC}x != x) then 
    if (! -e $M1_SP1) grppha $M1SPEC $M1_SP1 "chkey BACKFILE $M1_BGD & chkey RESPFILE $M1_RMF & chkey ANCRFILE $M1_ARF & bad 0-13 & good 14-40 & exit"
    if (! -e $M1_SP2) grppha $M1_SP1 $M1_SP2 "group min 25 & exit"
endif
if (${M2SPEC}x != x) then 
    if (! -e $M2_SP1) grppha $M2SPEC $M2_SP1 "chkey BACKFILE $M2_BGD & chkey RESPFILE $M2_RMF & chkey ANCRFILE $M2_ARF & bad 0-13 & good 14-40 & exit"
    if (! -e $M2_SP2) grppha $M2_SP1 $M2_SP2 "group min 25 & exit"
endif

#############
# (13) copy/move all source-specific files needed by XSPEC or for
# diagnostic purposes to work directory, then go there...
#
cp *BGSPEC0${SRCNUM_PPS}* ${XSPEC_WORKDIR}
cp *FIT ${XSPEC_WORKDIR}
cp *log ${XSPEC_WORKDIR}
cp ../../ppsSpecPrep.csh ${XSPEC_WORKDIR}
gzip -9v *FIT
cd ${XSPEC_WORKDIR}


#############
# (14) get the Galactic NH column
#
nh equinox=2000 ra=${SDSS_RA} dec=${SDSS_DEC} tchat=5 lchat=10 usemap=2 |& tee -a sas.log


#############
# (15) Set symbolic links to point to default set of spectra
#
# For now, I'll assume I want grouped spectra and not ungrouped ones.
# Need to implement a test for this...
#
ln -s ${PN_SP1} spec_PN_noGrp.fit
ln -s ${M1_SP1} spec_M1_noGrp.fit
ln -s ${M2_SP1} spec_M2_noGrp.fit
ln -s ${PN_SP2} spec_PN_grp25.fit
ln -s ${M1_SP2} spec_M1_grp25.fit
ln -s ${M2_SP2} spec_M2_grp25.fit

echo " " >> sas.log
echo "Finished pre-processing $IAUNAME_PPS (DETID $DETID_PPS, $COMMONNAME)" >> sas.log
echo " " >> sas.log
echo " " >> sas.log
echo " "
echo "Finished pre-processing $IAUNAME_PPS (DETID $DETID_PPS, $COMMONNAME)"
echo " "
popd
