#!/bin/tcsh
#
# Usage:
# source runEpatplot.csh EVTLIST SRCNUM "FILTER" LOGFILE
#
# This script extracts a source-only events list from EVTLIST using the filter
# parameters in FILTER (which may in general include any filter criteria: flags,
# regions, GTIs, etc).  "FILTER" must be enclosed in quotes because it will in
# general include space characters.  The provided SRCNUM is used in the naming
# of the output files.
#
# Verbose output is directed to LOGFILE and terse error and warning reports go
# to both LOGFILE and STDOUT.

echo " " >>! ${4}
echo "Now running specExtract.csh with the following parameters:" >>! ${4}
echo "    EVTLIST = ${1}" >>! ${4}
echo "    SRCNUM = ${2}" >>! ${4}
echo "    Filter = ${3}" >>! ${4}
echo "    LogFile = ${4}" >>! ${4}

set EVTlist_epat = ${1}
set SRCNUM_epat = ${2}
set FiltExpr_epat = "${3}"
set LogFile_epat = ${4}

fkeypar ${EVTlist_epat} INSTRUME
set INST_epat = `pget fkeypar value`
fkeypar ${EVTlist_epat} EXPIDSTR
set EXPID_epat = `pget fkeypar value`

if (-e tmp.out) /bin/rm -f tmp.out
set SrcEvLi = `echo ${EVTlist_epat} | sed -e "s/MIEVLI/SREVLI/g" -e "s/PIEVLI/SREVLI/g" -e "s/0000/0${SRCNUM_epat}/g"`
# echo "   Any errors or warnings that happened when extracting source-only events list" |& tee -a ${LogFile_epat}
# echo "   from ${EVTlist_epat} appear on the following lines:" |& tee -a ${LogFile_epat}
echo "> <> < ${INST_epat} exposure ${EXPID_epat} source spectrum" |& tee -a ${LogFile_epat}
echo "       EVSELECT warnings and errors:" |& tee -a ${LogFile_epat}
evselect table=${EVTlist_epat} keepfilteroutput=yes withfilteredset=yes \
    expression="${FiltExpr_epat}" filteredset=${SrcEvLi} >>&! tmp.out
cat tmp.out >>! ${LogFile_epat}
# grep "\*\*" tmp.out |& tee -a ${LogFile_epat}
grep -i " warning " tmp.out |& tee -a ${LogFile_epat}
grep -i " error " tmp.out |& tee -a ${LogFile_epat}
/bin/rm -f tmp.out
# echo "   Any errors or warnings from EPATPLOT appear on the following lines:" |& tee -a ${LogFile_epat}
echo "       EPATPLOT warnings and errors:" |& tee -a ${LogFile_epat}
epatplot set=${SrcEvLi} withqdp=yes modifyinset=no >>&! tmp2.out
cat tmp2.out >>! ${LogFile_epat}
# grep "\*\*" tmp2.out |& tee -a ${LogFile_epat}
grep -i " warning " tmp2.out |& tee -a ${LogFile_epat}
grep -i " error " tmp2.out |& tee -a ${LogFile_epat}
grep " %PGPLOT, " tmp2.out |& tee -a ${LogFile_epat}
grep -i " fault" tmp2.out |& tee -a ${LogFile_epat}
/bin/rm -f tmp2.out

unset SrcEvLi
unset EVTlist_epat
unset SRCNUM_epat
unset FiltExpr_epat
unset LogFile_epat
