#!/bin/tcsh
#
# Usage:
# source setIMAGElist.csh "EVTLIST" "EXPLIST" INST OBSID BAND SUFFIX LOGFILE
#
# Given an input list of XMM events files EVTLIST (which *MUST* be
# enclosed in quotation marks), cycle through each and generate a
# list of images in the energy band BAND from the corresponding
# exposures in EXPLIST of the instrument INST and the observation
# OBSID.  The SUFFIX is used for file matching: it should be "F" if
# we are to include PPS-generated spectra, "FIT" if not.  Output is
# sent to both LOGFILE and STDOUT/STDERR.
#
# The exposures list EXPLIST (which also must be in quotes) could
# in principal be extracted from EVTLIST, but is provided as an
# argument because it has already been generated so there is no
# reason to do the work twice.  Likewise OBSID would not be
# necessary if I never use this script in a directory that contains
# spectra from multiple observations.
#
# The constructed list is stored in $IMGAElist
# The vector extractIMAGE indicates which images need to be extracted.

echo " " >>! ${7}
echo "Now running setIMAGElist.csh with the following parameters:" >>! ${7}
echo "    EVTlist = ${1}" >>! ${7}
echo "    EXPlist = ${2}" >>! ${7}
echo "    INST = ${3}" >>! ${7}
echo "    OBSID = ${4}" >>! ${7}
echo "    BAND = ${5}" >>! ${7}
echo "    SUFFIX = ${6}" >>! ${7}
echo "    LOGFILE = ${7}" >>! ${7}


echo " "
echo "setIMAGElist.csh for ${3} band ${5}:"

set IMAGElist = ( $1 )
set extractIMAGE = ( $1 )
set EXPlist_set = ( $2 )
set INST_set = $3
set OBSID_set = $4
set BAND_set = $5
set SUFFIX_set = $6
set LOGFILE_set = $7

set tmpCounter = 1
while (${tmpCounter} <= ${#IMAGElist})
    set tempIMAGElist = `/bin/ls -t P${OBSID_set}${INST_set}${EXPlist_set[$tmpCounter]}IMAGE_${BAND_set}000*.${SUFFIX_set}* \
	|& tr '/' '_' | sed "s/tcsh: _bin_ls: No match.//g"`
    if ("$tempIMAGElist"x != x) then
	set IMAGElist[$tmpCounter] = ${tempIMAGElist[1]}
	set extractIMAGE[$tmpCounter] = 0
	if (${#tempIMAGElist} > 1) then
	    echo "setIMAGElist.csh:"
	    echo "Note: more than one image file found for ${INST_set} exposure ${EXPlist_set[$tmpCounter]} in band ${BAND_set}" |& tee -a ${LOGFILE_set}
	    echo "      The full list is:" >>! ${LOGFILE_set}
	    /bin/ls -lt P${OBSID_set}${INST_set}${EXPlist_set[$tmpCounter]}IMAGE_${BAND_set}000*.${SUFFIX_set}* >>! ${LOGFILE_set}
	    echo "      Only the newest one (${tempIMAGElist[1]}) will be used" |& tee -a ${LOGFILE_set}
	endif
    else
	set tempName = `echo ${IMAGElist[$tmpCounter]} | sed -e "s/MIEVLI/PIEVLI/g" -e "s/PIEVLI/IMAGE_/g" -e "s/0000/${BAND_set}000/g" -e "s/FTZ/FIT/g"`
	set IMAGElist[$tmpCounter] = $tempName
	set extractIMAGE[$tmpCounter] = 1
	unset tempName
    endif
    @ tmpCounter++
    unset tempIMAGElist
end

unset tmpCounter
unset BAND_set
unset EXPlist_set
unset INST_set
unset OBSID_set
unset SRCNUM_set
unset SUFFIX_set
unset LOGFILE_set
