#!/bin/tcsh
#
# Usage:
# source specExtract.csh EVTLIST SPECNAME "FILTER" LOGFILE
#
# A spectrum of the name SPECNAME will be extracted from the events list EVTLIST,
# making use of the selection expression FILTER (which *MUST* be enclosed in
# quotes because it will usually include space characters).  Verbose feedback
# will be stored in LOGFILE and more terse remarks will directed to STDOUT.
#
# This script automatically detects which EPIC camera took the data and sets
# spectral binning parameters accordingly.

echo " " >>! ${4}
echo "Now running specExtract.csh with the following parameters:" >>! ${4}
echo "    EVTfile = ${1}" >>! ${4}
echo "    SPECfile = ${2}" >>! ${4}
echo "    SelExpr = ${3}" >>! ${4}
echo "    LogFile = ${4}" >>! ${4}

set EVTlist_specEx = ${1}
set SPECname_specEx = ${2}
set FiltExpr_specEx = "${3}"
set LogFile_specEx = ${4}

fkeypar ${EVTlist_specEx} INSTRUME
set INSTstr_specEx = `pget fkeypar value | tr -d "'" | tr -d '"' | sed -e "s/EPN/PN/g" -e "s/EMOS/M/g" | tr '[a-z]' '[A-Z]'`

if (${INSTstr_specEx} == PN) then
    set specBinSize_specEx = 5
    set specChanMin_specEx = 0
    set specChanMax_specEx = 20479
else
    set specBinSize_specEx = 15
    set specChanMin_specEx = 0
    set specChanMax_specEx = 11999
    if ((${INSTstr_specEx} != M1) && (${INSTstr_specEx} != M2)) then
	echo "specExtract.csh:" |& tee -a ${LogFile_specEx}
	echo " *** ERROR: could not identif the instrument corresponding to ${EVTlist_specEx}" |& tee -a ${LogFile_specEx}
	echo "     Spectral binning appropriate for MOS will be used." |& tee -a ${LogFile_specEx}
    endif
endif

if (-e tmp.out) /bin/rm -f tmp.out
evselect table=${EVTlist_specEx} withspectrumset=yes spectrumset=${SPECname_specEx} \
    energycolumn=PI spectralbinsize=${specBinSize_specEx} withspecranges=yes \
    specchannelmin=${specChanMin_specEx} specchannelmax=${specChanMax_specEx} writedss=true \
    expression="${FiltExpr_specEx}" >>&! tmp.out
cat tmp.out >>! ${LogFile_specEx}
# echo "Below are errors/warnings that happened when creating ${SPECname_specEx}:" \
#     |& tee -a ${LogFile_specEx}
echo "       EVSELECT warnings and errors:" |& tee -a ${LogFile_specEx}
# grep "\*\*" tmp.out |& tee -a ${LogFile_specEx}
grep -i " warning " tmp.out |& tee -a ${LogFile_specEx}
grep -i " error " tmp.out |& tee -a ${LogFile_specEx}
/bin/rm -f tmp.out

unset EVTlist_specEx
unset SPECname_specEx
unset FiltExpr_specEx
unset LogFile_specEx
unset INSTstr_specEx
unset specBinSize_specEx
unset specChanMin_specEx
unset specChanMax_specEx
